@@ -1,7 +1,12 @@
 # -*- coding: utf-8 -*-
 
+import cgi
 import json
 import urllib
+try:
+    from urllib import urlencode
+except ImportError:
+    from urllib.parse import urlencode # python3
 try:
     import urlparse # python2
 except ImportError:
@@ -20,6 +25,10 @@ def parser_remove_null_expires_in(data):
         data.pop('expires_in')
     return data
 
+class UserNotFoundException(Exception):
+    pass
+
+
 @implementer(IOpenstaxAccounts)
 class OpenstaxAccounts(object):
 
@@ -55,6 +64,28 @@ class OpenstaxAccounts(object):
     def request(self, *args, **kwargs):
         return self.sanction_client.request(*args, **kwargs)
 
+    def search(self, query):
+        return self.request('/api/application_users.json?{}'.format(
+            urlencode({'q': query})))
+
+    def send_message(self, username, subject, body):
+        users = self.search('username:{}'.format(username))
+        userid = None
+        for user in users['users']:
+            if user['username'] == username:
+                userid = user['id']
+        if userid is None:
+            raise UserNotFoundException('User "{}" not found'.format(username))
+
+        self.request('/api/messages.json', data=urlencode({
+                     'user_id': int(userid),
+                     'to[user_ids][]': [int(userid)],
+                     'subject': subject,
+                     'body[text]': body,
+                     'body[html]': '<html><body>{}</body></html>'.format(
+                         cgi.escape(body)),
+                     }, True))
+
 
 def main(config):
     settings = config.registry.settings
@@ -69,5 +100,5 @@ def main(config):
     openstax_accounts.request_application_token()
     config.registry.registerUtility(openstax_accounts, IOpenstaxAccounts)
 
-    config.registry.registerUtility(OpenstaxAccounts(*args), IOpenstaxAccounts,
-            'authentication')
+    config.registry.registerUtility(OpenstaxAccounts, IOpenstaxAccounts,
+            name='factory')
